import sys
import os
os.environ["adaptive_binning_chisquared_2sam"]=os.getcwd()
#sys.path.insert(0, './GoF')

from chi2_regular_binning import *
from chi2_adaptive_binning import *
from chi2_plots import *
from calling_chi2_example import *

